@@ -1,3 +1,5 @@
+//    Copyright 2017, L. Hüdepohl and A. Marek, MPCDF
+//
 //    This file is part of ELPA.
 //
 //    The ELPA library was originally created by the ELPA consortium,
@@ -51,7 +53,7 @@
 
 #include "config.h"
 
-#ifdef WITH_OPENMP
+#ifdef WITH_OPENMP_TRADITIONAL
 #include <omp.h>
 #endif
 
@@ -131,6 +133,9 @@ static int skewsymmetric_is_valid(elpa_index_t index, int n, int new_value);
 
 static int is_positive(elpa_index_t index, int n, int new_value);
 
+static int elpa_float_string_to_value(char *name, char *string, float *value);
+static int elpa_float_value_to_string(char *name, float value, const char **string);
+
 static int elpa_double_string_to_value(char *name, char *string, double *value);
 static int elpa_double_value_to_string(char *name, double value, const char **string);
 
@@ -250,7 +255,7 @@ static const elpa_index_int_entry_t int_entries[] = {
 
         INT_ENTRY("max_stored_rows", "Maximum number of stored rows used in ELPA 1 backtransformation, default 63", 63, ELPA_AUTOTUNE_EXTENSIVE, ELPA_AUTOTUNE_DOMAIN_ANY, \
                         max_stored_rows_cardinality, max_stored_rows_enumerate, max_stored_rows_is_valid, NULL, PRINT_YES),
-#ifdef WITH_OPENMP
+#ifdef WITH_OPENMP_TRADITIONAL
         INT_ENTRY("omp_threads", "OpenMP threads used in ELPA, default 1", 1, ELPA_AUTOTUNE_FAST, ELPA_AUTOTUNE_DOMAIN_ANY, \
                         omp_threads_cardinality, omp_threads_enumerate, omp_threads_is_valid, NULL, PRINT_YES),
 #else
@@ -269,14 +274,34 @@ static const elpa_index_int_entry_t int_entries[] = {
         BOOL_ENTRY("cannon_for_generalized", "Whether to use Cannons algorithm for the generalized EVP", 1, ELPA_AUTOTUNE_NOT_TUNABLE, 0, PRINT_YES),
 };
 
+#define READONLY_FLOAT_ENTRY(option_name, option_description) \
+        { \
+                BASE_ENTRY(option_name, option_description, 0, 1, 0) \
+        }
+
+#define FLOAT_ENTRY(option_name, option_description, default, print_flag) \
+        { \
+                BASE_ENTRY(option_name, option_description, 0, 0, print_flag), \
+                .default_value = default, \
+        }
+
+static const elpa_index_float_entry_t float_entries[] = {
+        FLOAT_ENTRY("thres_pd_single", "Threshold to define ill-conditioning, default 0.00001", 0.00001, PRINT_YES),
+};
+
 #define READONLY_DOUBLE_ENTRY(option_name, option_description) \
         { \
                 BASE_ENTRY(option_name, option_description, 0, 1, 0) \
         }
 
+#define DOUBLE_ENTRY(option_name, option_description, default, print_flag) \
+        { \
+                BASE_ENTRY(option_name, option_description, 0, 0, print_flag), \
+                .default_value = default, \
+        }
+
 static const elpa_index_double_entry_t double_entries[] = {
-        /* Empty for now */
-        READONLY_DOUBLE_ENTRY("dummy", "dummy"),
+        DOUBLE_ENTRY("thres_pd_double", "Threshold to define ill-conditioning, default 0.00001", 0.00001, PRINT_YES),
 };
 
 void elpa_index_free(elpa_index_t index) {
@@ -536,6 +561,23 @@ int elpa_int_string_to_value(char *name, char *string, int *value) {
         return ELPA_ERROR_ENTRY_INVALID_VALUE;
 }
 
+int elpa_float_string_to_value(char *name, char *string, float *value) {
+        float val;
+        int ret = sscanf(string, "%lf", &val);
+        if (ret == 1) {
+                *value = val;
+                return ELPA_OK;
+        } else {
+                /* \todo: remove */
+                fprintf(stderr, "ELPA: DEBUG: Could not parse float value '%s' for option '%s'\n", string, name);
+                return ELPA_ERROR_ENTRY_INVALID_VALUE;
+        }
+}
+
+int elpa_float_value_to_string(char *name, float value, const char **string) {
+        return ELPA_ERROR_ENTRY_NO_STRING_REPRESENTATION;
+}
+
 int elpa_double_string_to_value(char *name, char *string, double *value) {
         double val;
         int ret = sscanf(string, "%lf", &val);
@@ -951,7 +993,7 @@ static int stripewidth_complex_is_valid(elpa_index_t index, int n, int new_value
 
 static int omp_threads_cardinality(elpa_index_t index) {
 	int max_threads;
-#ifdef WITH_OPENMP
+#ifdef WITH_OPENMP_TRADITIONAL
 	if (set_max_threads_glob == 0) {
 		max_threads_glob = omp_get_max_threads();
 		set_max_threads_glob = 1;
@@ -970,7 +1012,7 @@ static int omp_threads_enumerate(elpa_index_t index, int i) {
 
 static int omp_threads_is_valid(elpa_index_t index, int n, int new_value) {
         int max_threads;
-#ifdef WITH_OPENMP
+#ifdef WITH_OPENMP_TRADITIONAL
 	if (set_max_threads_glob == 0) {
 		max_threads_glob = omp_get_max_threads();
 		set_max_threads_glob = 1;
