@@ -1,3 +1,53 @@
+#if 0
+!    Copyright 2016, P. Kus
+!
+!    This file is part of ELPA.
+!
+!    The ELPA library was originally created by the ELPA consortium,
+!    consisting of the following organizations:
+!
+!    - Max Planck Computing and Data Facility (MPCDF), formerly known as
+!      Rechenzentrum Garching der Max-Planck-Gesellschaft (RZG),
+!    - Bergische Universität Wuppertal, Lehrstuhl für angewandte
+!      Informatik,
+!    - Technische Universität München, Lehrstuhl für Informatik mit
+!      Schwerpunkt Wissenschaftliches Rechnen ,
+!    - Fritz-Haber-Institut, Berlin, Abt. Theorie,
+!    - Max-Plack-Institut für Mathematik in den Naturwissenschaften,
+!      Leipzig, Abt. Komplexe Strukutren in Biologie und Kognition,
+!      and
+!    - IBM Deutschland GmbH
+!
+!    This particular source code file contains additions, changes and
+!    enhancements authored by Intel Corporation which is not part of
+!    the ELPA consortium.
+!
+!    More information can be found here:
+!    http://elpa.mpcdf.mpg.de/
+!
+!    ELPA is free software: you can redistribute it and/or modify
+!    it under the terms of the version 3 of the license of the
+!    GNU Lesser General Public License as published by the Free
+!    Software Foundation.
+!
+!    ELPA is distributed in the hope that it will be useful,
+!    but WITHOUT ANY WARRANTY; without even the implied warranty of
+!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+!    GNU Lesser General Public License for more details.
+!
+!    You should have received a copy of the GNU Lesser General Public License
+!    along with ELPA.  If not, see <http://www.gnu.org/licenses/>
+!
+!    ELPA reflects a substantial effort on the part of the original
+!    ELPA consortium, and we ask you to respect the spirit of the
+!    license that we chose: i.e., please contribute any changes you
+!    may have back to the original ELPA library distribution, and keep
+!    any derivatives of ELPA under the same license that we chose for
+!    the original distribution, the GNU Lesser General Public License.
+!
+!
+#endif
+
 #ifdef REALCASE
 #undef DOUBLE_PRECISION_REAL
 #undef SINGLE_PRECSION_REAL
@@ -64,7 +114,6 @@
 #undef  MPI_MATH_DATATYPE_PRECISION_C
 #undef  MPI_MATH_DATATYPE_PRECISION_EXPL
 #undef  C_DATATYPE_KIND
-#undef THRESHOLD
 
 
 #if 0
@@ -138,7 +187,6 @@
 #define C_PLACPY pdlacpy_
 #define C_PTRAN pdtran_
 
-#define THRESHOLD 1e-11_rk8
 #endif /* DOUBLE_PRECISION */
 
 #ifdef SINGLE_PRECISION
@@ -206,7 +254,6 @@
 #define C_PLACPY pslacpy_
 #define C_PTRAN pstran_
 
-#define THRESHOLD 1e-4_rk4
 #endif /* SINGLE_PRECISION */
 
 #endif /* REALCASE */
@@ -290,7 +337,6 @@
 #undef ELPA_PRECISION_SSMV
 #undef ELPA_PRECISION_SSR2
 
-#undef THRESHOLD
 
 #if 0
 /* General definitions needed in single and double case */
@@ -369,8 +415,6 @@
 #define ELPA_PRECISION_SSMV elpa_zssmv
 #define ELPA_PRECISION_SSR2 elpa_zssr2
 
-
-#define THRESHOLD 1e-11_rk8
 #endif /* DOUBLE PRECISION */
 
 #ifdef SINGLE_PRECISION
@@ -442,7 +486,6 @@
 #define ELPA_PRECISION_SSMV elpa_cssmv
 #define ELPA_PRECISION_SSR2 elpa_cssr2
 
-#define THRESHOLD 1e-4_rk4
 #endif /* SINGLE PRECISION */
 
 #endif /* COMPLEXCASE */
